@@ -235,7 +235,8 @@ func (l *lexer) Lex(lval *yySymType) (tokenType int) {
 	default:
 		if ch >= utf8.RuneSelf {
 			r, size := utf8.DecodeRuneInString(l.source[l.offset-1:])
-			l.offset += size
+			// -1 to adjust for first byte consumed by next()
+			l.offset += size - 1
 			l.token = string(r)
 		}
 	}
@@ -247,15 +248,9 @@ func (l *lexer) next() (byte, bool) {
 		ch := l.source[l.offset]
 		l.offset++
 		if ch == '#' {
-			if len(l.source) == l.offset {
+			if l.skipComment() {
 				return 0, true
 			}
-			for !isNewLine(l.source[l.offset]) {
-				l.offset++
-				if len(l.source) == l.offset {
-					return 0, true
-				}
-			}
 		} else if !isWhite(ch) {
 			return ch, false
 		} else if len(l.source) == l.offset {
@@ -264,6 +259,28 @@ func (l *lexer) next() (byte, bool) {
 	}
 }
 
+func (l *lexer) skipComment() bool {
+	for {
+		switch l.peek() {
+		case 0:
+			return true
+		case '\\':
+			switch l.offset++; l.peek() {
+			case '\\', '\n':
+				l.offset++
+			case '\r':
+				if l.offset++; l.peek() == '\n' {
+					l.offset++
+				}
+			}
+		case '\n', '\r':
+			return false
+		default:
+			l.offset++
+		}
+	}
+}
+
 func (l *lexer) peek() byte {
 	if len(l.source) == l.offset {
 		return 0
@@ -490,7 +507,7 @@ func quoteAndEscape(src string, quote bool, controls int) []byte {
 		buf[len(buf)-1] = '"'
 		j++
 	}
-	for i := 0; i < len(src); i++ {
+	for i := range len(src) {
 		if ch := src[i]; ch < ' ' {
 			const hex = "0123456789abcdef"
 			copy(buf[j:], `\u00`)
@@ -505,37 +522,34 @@ func quoteAndEscape(src string, quote bool, controls int) []byte {
 	return buf
 }
 
-type parseError struct {
-	offset    int
-	token     string
+// ParseError represents a description of a query parsing error.
+type ParseError struct {
+	Offset    int    // the error occurred after reading Offset bytes
+	Token     string // the Token that caused the error (may be empty)
 	tokenType int
 }
 
-func (err *parseError) Error() string {
+func (err *ParseError) Error() string {
 	switch err.tokenType {
 	case eof:
 		return "unexpected EOF"
 	case tokInvalid:
-		return "invalid token " + jsonMarshal(err.token)
+		return "invalid token " + jsonMarshal(err.Token)
 	case tokInvalidEscapeSequence:
-		return `invalid escape sequence "` + err.token + `" in string literal`
+		return `invalid escape sequence "` + err.Token + `" in string literal`
 	case tokUnterminatedString:
 		return "unterminated string literal"
 	default:
-		return "unexpected token " + jsonMarshal(err.token)
+		return "unexpected token " + jsonMarshal(err.Token)
 	}
 }
 
-func (err *parseError) Token() (string, int) {
-	return err.token, err.offset
-}
-
 func (l *lexer) Error(string) {
 	offset, token := l.offset, l.token
 	if l.tokenType != eof && l.tokenType < utf8.RuneSelf {
 		token = string(rune(l.tokenType))
 	}
-	l.err = &parseError{offset, token, l.tokenType}
+	l.err = &ParseError{offset, token, l.tokenType}
 }
 
 func isWhite(ch byte) bool {
@@ -562,12 +576,3 @@ func isHex(ch byte) bool {
 func isNumber(ch byte) bool {
 	return '0' <= ch && ch <= '9'
 }
-
-func isNewLine(ch byte) bool {
-	switch ch {
-	case '\n', '\r':
-		return true
-	default:
-		return false
-	}
-}
